<?php
namespace Teknisa\FIS\Modules\Acessorias\Services;

use Teknisa\FIS\Modules\Acessorias\Factories\Fis01600SintegraFactory;
use Teknisa\FIS\Modules\Acessorias\Services\Fis91000ObrigacoesAcessoriasService;
use Teknisa\FIS\Modules\Acessorias\Services\Agendamento;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use \Teknisa\Libs\Util\Libraries\PHPMailer;

class Fis01600SintegraService {

    protected $environment;
    protected $sintegraFactory;
    protected $obrigacoesService;
    
    public $mensagemBody = '';
    public $mensagemBodySucess = '';
    public $mensagemError = '';
    public $mensagemErrorBody = '';
    public $geraArquivo = '';

    public function __construct(Environment $environment, Fis01600SintegraFactory $sintegraFactory, Fis91000ObrigacoesAcessoriasService $obrigacoesService,
                                    NewCode $newCode, Agendamento $agendamento ) 
    {
        $this->environment              = $environment;
        $this->sintegraFactory          = $sintegraFactory;
        $this->obrigacoesService        = $obrigacoesService;
        $this->newCode                  = $newCode;
        $this->agendamento              = $agendamento;
    }

    /*
     * Função para criar o arquivo da exportação do SINTEGRA.
     */
    private function createFileSintegra($row) {
        // Cria os parâmetros para buscar os dados gerados na procedure.
        $params = array(
            "NRORG" => $row["NRORG"],
            "NRSEQARQMAG" => $row["NRSEQARQMAG"]
        );
        $sintegraData = $this->sintegraFactory->getSintegraData($params);
        
        if(isset($row['NRAGENDAMENTO']) && empty($sintegraData)){
            $this->mensagemError = '<br />'."Não há dados a serem exibidos.";
            return $fileName = false;
        }
        // Chama a função para a criação do arquivo.
        $fileName = $this->obrigacoesService->createFileExportacao($row["NMARQUIVO"], $sintegraData, "DSREGARQMAG");

        // Retorna o nome do arquivo exportado.
        return $fileName;
    }

    /*
     * Função para exportar o SINTEGRA.
     */
    public function exportarSintegra($row) {
        // Valida os dados enviados na requisição.
        $this->validateSintegra($row);
        
        // Gera o próximo código sequencial do arquivo magnético.
        $row["NRSEQARQMAG"] = $this->obrigacoesService->getNextNrseqarqmag($row["NRORG"]);

        // Monta o array com os parâmetros para chamar a procedure.
        $params = array(
            "CDIMPOSTO"     => '00',
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "CDINSCESTAST"  => empty($row["CDINSCESTAST"]) ? null : $row["CDINSCESTAST"],
            "DTINICIAL"     => $row["DTINICIAL"],
            "DTFINAL"       => $row["DTFINAL"],
            "DTPOSICAO"     => null,
            "VERSAO"        => $row["VERSAO"],
            "SGESTADO"      => $row["SGESTADO"],
            "FINALIDADE"    => $row["FINALIDADE"],
            "NRSEQARQMAG"   => $row["NRSEQARQMAG"],
            "STARQEXP"      => null,
            "INVENTARIO"    => 'S',
            "GERAFORSALE"   => 'N'
        );

        // Chama a procedure de exportação.
        $this->sintegraFactory->callExportacaoProcedure($params);

        // Foi evitado utilizar um bloco finally devido a possíveis
        // incompatibilidades com a versão do PHP.
        try{
            // Seta os parâmetros para exclusão dos dados temporários.
            $params = array(
                "NRORG" => $row["NRORG"],
                "NRSEQARQMAG" => $row["NRSEQARQMAG"]
            );
            // Chama a função para gerar o arquivo texto.
            $fileName = $this->createFileSintegra($row);
            // Exclui os dados temporários da exportação no banco de dados.
            $this->sintegraFactory->deleteSintegraData($params);

            // Retorna o nome do arquivo com suas possíveis modificações.
            return $fileName;
        } catch(\Exception $e) {
            // Exclui os dados temporários da exportação no banco de dados.
            $this->sintegraFactory->deleteSintegraData($params);
            throw $e;
        }
    }

    /*
     * Função para validação dos dados informados para o SINTEGRA.
     */
    private function validateSintegra($row) {
        if (isset($row['NRAGENDAMENTO'])){
            
            $this->mensagemError = '';
            
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Empresa deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
            if (empty($row["CDINSCESTA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Inscrição Estadual deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Inscrição Estadual deve ser informada.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Referência deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Identificação do Convênio" possui algum valor informado.
            if (empty($row["VERSAO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Identificação do Convênio deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Identificação do Convênio deve ser informada.");
            }
            // Verifica se o campo "Finalidade" possui algum valor informado.
            if (empty($row["FINALIDADE"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Finalidade deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Finalidade deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Nome do Arquivo deve ser informado.';
                $this->geraArquivo = false;
                //throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
            
            // Seta o imposto a ser validado no mapa de apuração.
            $impostos = array('00');
    
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Todos os mapas devem estar bloqueados para geração do arquivo.';
                $this->geraArquivo = false;
                //throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }
        } else {
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Inscrição Estadual" possui algum valor informado.
            if (empty($row["CDINSCESTA"])) {
                throw new \Exception("Inscrição Estadual deve ser informada.");
            }
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                throw new \Exception("Referência deve ser informada.");
            }
            // Verifica se o campo "Identificação do Convênio" possui algum valor informado.
            if (empty($row["VERSAO"])) {
                throw new \Exception("Identificação do Convênio deve ser informada.");
            }
            // Verifica se o campo "Finalidade" possui algum valor informado.
            if (empty($row["FINALIDADE"])) {
                throw new \Exception("Finalidade deve ser informada.");
            }
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            $nrorg = $this->environment->getNrorg();
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
            $row["NRORG"] = $nrorg;
    
            // Seta o imposto a ser validado no mapa de apuração.
            $impostos = array('00');
    
            // Valida o mapa de apuração.
            $mapaApuracIsValid = $this->obrigacoesService->mapaApuracIsValid($row, $impostos);
    
            // Verifica se o mapa está inválido, lançando um erro em caso positivo.
            if (!$mapaApuracIsValid) {
                throw new \Exception("Todos os mapas devem estar bloqueados para geração do arquivo.");
            }
        }
    }
    
    public function findAgendamento() {
        // Busca os parâmetros conforme empresa e nrorg.
        $agendamento = $this->sintegraFactory->findAgendamento();

        return $agendamento;
    }
    
    public function saveAgendamento($row) {
        try {
            
            $row['CDTELA'] = 'FIS01600';
            
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes($this->formataData($row['DTREFINI']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes($this->formataData($row['DTREFINI']));
            $dtinicial = $dtinicial->format("d/m/Y");
            $dtfinal = $dtfinal->format("d/m/Y");
    
            $row["DTINICIAL"] = $dtinicial;
            $row["DTFINAL"] = $dtfinal;
            
            //var_dump($row);
            //throw new \Exception('ioioioioioi');
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $row["NRAGENDAMENTO"] = $this->newCode->getCodeByTableName('NRAGENDAMENTO', $row["NRORG"], 1, 10);
            }
            
            if($row['CDINSCESTA'] == 'T'){
                $arrayInscEsta = $this->sintegraFactory->getInscEsta($row);
                $cdinscesta = '_';
                
                foreach($arrayInscEsta as $value){
                    $cdinscesta = $cdinscesta.$value['CDINSCESTA'].'_';
                }
                
                $row['CDINSCESTA'] = $cdinscesta;
            } else if(is_array($row['CDINSCESTA']) == true){
                $arrayInscEsta = $row['CDINSCESTA'];
                $cdinscesta = '_';
                
                foreach($arrayInscEsta as $value){
                    $cdinscesta = $cdinscesta.$value.'_';
                }
                
                $row['CDINSCESTA'] = $cdinscesta;
            } 
        
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $this->sintegraFactory->insertAgendamento($row);
            } else {
                $this->sintegraFactory->updateAgendamento($row);
                
                 $arquivos = $this->sintegraFactory->getAgendamentoArquivo($row);
                
                if($arquivos){
                    foreach($arquivos as $arquivo){
                        unlink($arquivo['NMPATHARQ']);
                    }
                    
                    $this->sintegraFactory->deleteAgendamentoArquivo($arquivo);
                }
                
                $idstatus = 'P';
                $this->sintegraFactory->updateAgendamentoIdStatus($row, $idstatus);
                
            }
            
            $agendamento = $this->sintegraFactory->getAgendamentoInsert($row);
            
            if($agendamento){
                
                $agendamento['NRORG'] = $row['NRORG'];
                $getConfigSMTP = $this->sintegraFactory->getConfiguraSMTP($agendamento)[0];
                if(isset($row['__is_new']) && $row['__is_new'] == true){
                    $nomeJob = $row['CDTELA'].'_'.$row['NRAGENDAMENTO'];
                } else {
                    $nomeJob = $row['CDTELA'].'_'.str_pad($row['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                }
                $dataContabAuto = $row['DTAGENDAMENTO'];
                $sequencialJob = $row['NRAGENDAMENTO'];
                $nrorg = $row['NRORG'];
                $descricao = $row['CDTELA'].' - SINTEGRA (Estadual)'.'. Exportação das declarações Acessórias de forma automática na data de '.$row['DTAGENDAMENTO'];
                //Conferir caminho do diretorio do xampp
                $arqCallService = $getConfigSMTP['CALLSERVPATH'];//'C:\Teknisa\CallService.php';
                //$dirphp = str_replace('/','\',$DIRPHP);
                $dirphp = $getConfigSMTP['DIRPHP'];//'C:/xampp/php';
                $jenkinsPath = $getConfigSMTP['JENKINSPATH'];//'http://192.168.120.132:8080/';
                $execPath = '/execAgendamentoSintegra';
                $periodicidade = $row['IDPERIAGEN'];
                $jenkinsCreateItem = $getConfigSMTP['JENKCREATITEM'];
                $this->agendamento->criaJob($nomeJob, $dataContabAuto, $sequencialJob, $nrorg, $descricao, $arqCallService, $dirphp, $jenkinsPath, $execPath, $periodicidade, $jenkinsCreateItem);
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    public function execAgendamento($row){
        try{
            $agendamento = $this->sintegraFactory->findAgendamentoExec($row)[0];
            
            if($agendamento){
                $this->mensagemBody = '';
                
                $agendamento["NRORG"] = $row['NRORG'];
                
                //Necessário arrumar a data
                $agendamento["DTREFINI"] = explode(' ', $agendamento["DTREFINI"])[0];
                
                $agendamento["DTEXECAGEND"] = $agendamento["DTATUAL"];
                
                $agendamento['CDINSCESTAARRAY'] = $agendamento['CDINSCESTA'];
                $arrayInscEsta = explode('_',$agendamento['CDINSCESTAARRAY']);
                
                $getConfigSMTP = $this->sintegraFactory->getConfiguraSMTP($agendamento)[0];
                $getContatos = $this->sintegraFactory->getContatos($agendamento);
                
                $mail = new PHPMailer;
                $mail->CharSet = 'UTF-8';
                $mail->isSMTP();                        // Set mailer to use SMTP
                $mail->isHTML();
                $mail->Host = $getConfigSMTP['SERVIDORSMTP'];       // Specify main and backup SMTP servers
                $mail->SMTPAuth = true;                 // Enable SMTP authentication
                $mail->Username = $getConfigSMTP['LOGINSMTP'];    // SMTP username
                $mail->Password = $getConfigSMTP['SENHASMTP'];    // SMTP password
                $mail->From = $getConfigSMTP['LOGINSMTP'];
                $mail->FromName = 'ERP Pebbian';
                $mail->Subject = 'Sintegra (Estadual) de '.$agendamento['DTAGENDAMENTO'];
                // $mail->addAddress($cliente['DSEMAILCLIE']);
                // $mail->addAddress($vendedor['DSEMAILOPER']);
                
                foreach($getContatos as $contato){
                    $mail->addAddress($contato['EMAIL']);
                };
                
                $agendamento["NMEMPRESA"] = $this->sintegraFactory->getNmEmpresa($agendamento)["NMEMPRESA"];
                        
                $this->mensagemBody = 'Caro cliente, '.'<br />'.'<br />';
                $this->mensagemBodySucess = 'Arquivo de Sintegra (Estadual) gerado com Sucesso!'
                .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                $this->mensagemErrorBody = 'Erro ao gerar o arquivo de Sintegra (Estadual):'.'<br />'.'<br />'
                .'<br />'.'Empresa: '.$agendamento['CDEMPRESA'].' - '.$agendamento['NMEMPRESA'];
                
                
                $contErro = 0;
                $contSucess = 0;
                
                foreach( $arrayInscEsta as $inscEsta ){
                    if(isset($inscEsta) && $inscEsta != ''){
                        
                        $this->geraArquivo = true;
                            
                        $dataName = str_replace('/', '', explode(' ', $agendamento["DTATUAL"])[0]);
                        $horaName = str_replace(':', '', explode(' ', $agendamento["DTATUAL"])[1]);
                        
                        if($agendamento['IDPERIAGEN'] == '0'){
                            $dtinicial = $agendamento["DTINICIAL"];
                            $dtfinal = $agendamento["DTFINAL"];
                        } 
                        else if($agendamento['IDPERIAGEN'] == '1'){
                            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["DTATUAL"])[0]));
                            $dtinicial = $dtinicial->format("d/m/Y");
                            $dtfinal = $dtfinal->format("d/m/Y");
                        } 
                        else if($agendamento['IDPERIAGEN'] == '2'){
                            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData(explode(' ', $agendamento["LAST_MONTH"])[0]));
                            $dtinicial = $dtinicial->format("d/m/Y");
                            $dtfinal = $dtfinal->format("d/m/Y");
                        }
                        $agendamento["DTINICIAL"] = $dtinicial;
                        $agendamento["DTFINAL"] = $dtfinal;
                        
                        $agendamento['CDINSCESTA'] = $inscEsta;
                        $agendamento['SGESTADO'] = $this->sintegraFactory->getSgEstado($agendamento['CDINSCESTA'])['SGESTADO'];
                        $agendamento["NMARQUIVO"] = $agendamento['CDTELA'].$agendamento['NRAGENDAMENTO'].'_'.$inscEsta.'_'.$dataName.$horaName;
                          
                        $fileName = $this->exportarSintegra($agendamento);
                        
                        if($this->mensagemError == ''){
                            $this->mensagemBodySucess = $this->mensagemBodySucess
                            .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA'];
                            ++$contSucess;
                            
                        } else {
                            $this->mensagemErrorBody = $this->mensagemErrorBody
                            .'<br />'.'Inscrição Estadual: '.$agendamento['CDINSCESTA']
                            .$this->mensagemError.'<br />'.'<br />';
                            ++$contErro;
                        }
                        
                        if($fileName){
                            $file = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName;
                            $agendamento["NMPATHARQ"] = $file ;
                            
                            $verifArq = $this->sintegraFactory->verAgendamentoArquivo($agendamento);
                            
                            if(empty($verifArq)){
                                $agendamento["NRARQUIVO"] = $this->newCode->getCodeByTableName('NRARQUIVO', $agendamento["NRORG"], 1, 10);
                                $this->sintegraFactory->insertAgendamentoArquivo($agendamento);
                            }
                        }
                
                    }
                }
                
                if(($contErro > 0) && ($contSucess > 0)){
                    $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'.'<br />'
                    .'---------------------------------------------------------------------'.'<br />'
                    .$this->mensagemErrorBody
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                } 
                else if($contErro > 0){
                    $mail->Body = $this->mensagemBody.'<br />'
                    .$this->mensagemErrorBody
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                }
                else if($contSucess > 0){
                    $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'
                    .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                    $mail->send();
                }
                
                $idstatus = 'R';
                $this->sintegraFactory->updateAgendamentoIdStatus($agendamento, $idstatus);
                
            }
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteAgendamento($row){
        try {    
            
            $agendamento = $this->sintegraFactory->getAgendamentoInsert($row)[0];
            
            
            if($agendamento){
                $nomeJob = $agendamento['CDTELA'].'_'.str_pad($agendamento['NRAGENDAMENTO'], 10, "0", STR_PAD_LEFT);
                $getConfigSMTP = $this->sintegraFactory->getConfiguraSMTP($agendamento)[0];
                
                $this->agendamento->apagaJob($nomeJob, null, $getConfigSMTP['JENKINSPATH'], $getConfigSMTP["CALLSERVPATH"]);
                
                $arquivos = $this->sintegraFactory->getAgendamentoArquivo($agendamento);
                
                if($arquivos){
                    foreach($arquivos as $arquivo){
                        unlink($arquivo['NMPATHARQ']);
                    }
                    
                    $this->sintegraFactory->deleteAgendamentoArquivo($arquivo);
                    $this->sintegraFactory->deleteAgendamento($arquivo);
                    
                } else {
                    $this->sintegraFactory->deleteAgendamento($agendamento);
                }
                
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
                
    }
 
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
}